//
//  GlowWorm.h
//  GlowWorm
//
//  Created by Curtis Jones on 2006.06.13.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#ifndef __GLOW_WORM_H__
#define __GLOW_WORM_H__

#import <Cocoa/Cocoa.h>

#import <errno.h>
#import <sys/ioctl.h>
#import <sys/types.h>
#import <sys/ioccom.h>
#import <sys/socket.h>
#import <sys/stat.h>
#import <sys/sys_domain.h>
#import <sys/kern_control.h>
#import <assert.h>
#import <sys/uio.h>
#import <unistd.h>
#import <fcntl.h>
#import <semaphore.h>
#import <sys/sysctl.h>
#import <mach/mach.h>
#import <mach/task_info.h>
#import <kvm.h>
#import <paths.h>
#import <dirent.h>
#import <mach-o/dyld.h>
#import <pthread.h>
#import <sys/un.h>
#import <errno.h>

#include <Security/Authorization.h>
#include <Security/AuthorizationTags.h>

#import <DNSCacheEntry.h>
#import <Transaction.h>
#import <EasyTrans.h>
#import <IntfTrans.h>
#import <LogrTrans.h>
#import <ProcTrans.h>
#import <RegcTrans.h>
#import <RuleTrans.h>
#import <SockTrans.h>
#import <GlowWormProc.h>
#import <GlowWormRule.h>
#import <GlowWormRegc.h>
#import <GlowWormSock.h>

#define GW_DIRECTION_INCOMING				0x00000001
#define GW_DIRECTION_OUTGOING				0x00000002
#define GW_DIRECTION_ANY						0x00000003

#define GW_ACTION_BLOCK							0x00000001
#define GW_ACTION_CALLBACK					0x00000002


/*!
	@header GlowWorm
	@discussion The GlowWorm class is the base for all functionality. Only a single instance of this
		class should exist at a time. You should never have to manually create this instance.
		
		In the future, each GlowWorm instance will represent a connection to a specific machine, and
		multiple instances will be allowed. Not at the moment, though.
*/
@interface GlowWorm : NSObject { }

/*!
	@function getInstance
	@abstract Returns the singleton GlowWorm instance.
	@discussion Only a single instance of GlowWorm should exist at a time. The creation of this single
		instance is handled internally. Just call this class method to get it.
	@result Return:
		GlowWorm - The single static instance of GlowWorm for this runtime.
		NULL - Something went horribly wrong.
*/
+ (GlowWorm *)getInstance;

/*!
	@function initLocalUserList
	@abstract Retrieves the local user list from NetInfo.
	@discussion If you are writing a plugin, you do not ever need to call this method. If you are
		writing a stand-alone application, this method needs to be called before the localUsers method
		will be able to return the local user list.
		
		Ultimately, this mess will be handled through the kernel extension, so that the local user list
		will pertain to the machine running the kernel extension and not necessarily the machine running
		the application.
	@result Return:
		TRUE - Was able to retrieve the local user list.
		FALSE - Was not able to retrieve the local user list.
*/
- (BOOL)initLocalUserList;

/*!
	@function initLocalGroupList
	
	@abstract Retrieves the local group list from NetInfo.
	
	@discussion If you are writing a plugin, you do not ever need to call this method. If you are
		writing a stand-alone application, this method needs to be called before the localGroups method
		will be able to return the local group list.
		
		Ultimately, this mess will be handled through the kernel extension, so that the local group list
		will pertain to the machine running the kernel extension and not necessarily the machine running
		the application.
	
	@result Return:
		TRUE - Was able to retrieve the local group list.
		FALSE - Was not able to retrieve the local group list.
*/
- (BOOL)initLocalGroupList;

/*!
	@function localUsers
	
	@abstract Returns the local user list for this machine.
	
	@discussion Before this will return the local user list, the initLocalUserList method must be
		called. If you are writing a plugin, you do not need to worry about this. If you are writing
		a stand-alone application then you must make sure this is accomplished.
		
		The elements of the NSDictionary returned by this method are keyed on the respective uids 
		(NSNumber) of the users. Each object is also an NSDictionary with the keys "name", "realname"
		and "uid".
		
	@result Return:
		NSDictionary - The local user list.
		NULL - The initLocalUserList method was not called first or did not succeed.
*/
- (NSDictionary *)localUsers;
- (NSDictionary *)localGroups;

/* basic stuff */
- (BOOL)start:(AuthorizationRef *)anAuth;
- (BOOL)stop;
- (BOOL)isStarted;
- (BOOL)initRandom;
- (BOOL)connect;
- (BOOL)disconnect;
- (BOOL)loadKext:(AuthorizationRef *)anAuth;
- (BOOL)unloadKext:(AuthorizationRef *)anAuth;
- (void)setUnloadKextOnStop:(BOOL)aBool;

/* permission limitations */
- (void)resetPerms;
- (BOOL)processRegistration:(GlowWormRegc *)aRegc;
- (BOOL)isExpired;
- (uint32_t)permsSystExpires;
- (uint32_t)permsSystProduct;
- (uint32_t)permsRuleMaxAppl;
- (uint32_t)permsRuleMaxConn;
- (uint32_t)permsRuleMaxUid;
- (uint32_t)permsRuleMaxGid;
- (uint32_t)permsRuleMaxCnt;
- (uint32_t)permsRuleLog;
- (uint32_t)permsRuleCall;
- (uint32_t)permsConnDir;
- (uint32_t)permsConnProto;
- (BOOL)permsFops;





- (void)insertDnsLookupHandler:(id)callbackObj sel:(SEL)callbackSel;
- (void)insertDnsEntry:(DNSCacheEntry *)dce withBlocking:(BOOL)doBLocking callbackObj:(id)callbackObj callbackSel:(SEL)callbackSel;
- (void)selectDnsEntryWithBlocking:(BOOL)doBlocking callbackObj:(id)callbackObj callbackSel:(SEL)callbackSel;





- (void)connAuthAllow:(GlowWormMesg *)aMesg;
- (void)connAuthDeny:(GlowWormMesg *)aMesg;
- (void)connAuthDefer:(GlowWormMesg *)aMesg;
- (void)connAuth:(uint32_t)anAction mesg:(GlowWormMesg *)aMesg;

- (void)setDefaultHandler:(id)callbackObj callbackSel:(SEL)callbackSel;




- (pid_t)pidToPathDaemonProcessId;

- (void)readLoop;
- (void)procLoop;
- (void)sendLoop;

- (void)addLocalTransHandler:(Transaction *)aTrans;
- (void)delLocalTransHandler:(Transaction *)aTrans;
- (Transaction *)handlerById:(int)handlerid;
- (void)delAllLocalTransHandlers;

- (void)addTransToRecvQueue:(Transaction *)trans;
- (void)addTransToSendQueue:(Transaction *)trans;
- (Transaction *)getNextTransFromRecvQueue;
- (Transaction *)getNextTransFromSendQueue;
- (void)addTrans:(Transaction *)trans toQueue:(NSMutableArray *)queue withMaxSize:(int)maxsize andSemaphore:(sem_t *)qsem;

- (NSString *)getSocketErrorMessage:(int)error;

#pragma mark -
#pragma mark miscellaneous

- (BOOL)setLaunchDaemon:(NSString *)path runAtLoad:(BOOL)runAtLoad;

+ (BOOL)actionVerifySetUidForFile:(NSString *)path;
+ (BOOL)actionRepairSetUid:(AuthorizationRef *)anAuth file:(NSString *)path;

- (NSString *)pidToPath:(int)pid;
- (NSString *)pidToPathWithSock:(GlowWormSock *)aSock;
- (NSString *)pidToPathDaemon:(int)aPid;
- (NSString *)pidToPathKernel:(int)aPid;

+ (void)hexDump:(u_char *)buf len:(int)len;

+ (NSImage *)retainedHighlightedImageForImage:(NSImage *)image;
+ (void)runOpenWithArgument:(NSString *)anArg;
+ (void)doErrorAndQuit:(NSString *)aTitle aMessage:(NSString *)aMessage;

- (NSString *)portToServiceName:(int)aPort;

/* directory */
+ (BOOL)directoryExists:(NSString *)aString;
+ (BOOL)createDirectory:(NSString *)aString;
+ (BOOL)createDirectory:(NSString *)aString withAuth:(AuthorizationRef *)anAuth;
+ (BOOL)fileExists:(NSString *)aString;

/* authorization */
- (AuthorizationRef *)getAuthorization:(const char *)aName;
- (void)releaseAuthorization:(AuthorizationRef *)anAuthRef;
+ (BOOL)isEffectivelyRoot;
- (BOOL)runAsRoot:(char **)args;
- (int)runNormally:(char **)args;
- (BOOL)killAsRoot:(NSString *)path;
- (BOOL)killAsRoot:(NSString *)path withSignal:(int)aSignal;

/* system status */
+ (NSTimeInterval)uptime;
+ (NSDate *)bootTime;

/* icon stuff */
//+ (NSImage *)getApplicationIcon:(NSString *)aString withProcessID:(int)pid isFirst:(BOOL)aBool;
+ (NSImage *)getGenericApplicationIcon;
+ (NSImage *)getGenericExecutableIcon;
+ (NSString *)getApplicationShortName:(NSString *)aString;
+ (NSImage *)getApplicationIcon:(NSString *)aString;
+ (NSString *)dateToString:(NSDate *)aDate;

+ (NSDictionary *)getLocalUserList;
+ (NSDictionary *)getLocalGroupList;

@end

#endif /* __GLOW_WORM_H__ */
